from unittest import mock

from django.test import TestCase
from jsm_user_services import local_threading
from jsm_user_services.services.user import current_jwt_token
from jsm_user_services.services.user import get_jsm_token
from jsm_user_services.services.user import get_jsm_user_data_from_jwt
from jsm_user_services.services.user import get_ltm_token
from jsm_user_services.services.user import get_ltm_user_data_from_jwt
from jsm_user_services.services.user import get_user_email_from_jwt
from jsm_user_services.services.user import jwt_has_required_roles


class TestContracts(TestCase):
    def setUp(self):

        self.jwt = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJPbmxpbmUgSldUIEJ1aWxkZXIiLCJpYXQiOjE1NTkxNzc3MTcsImV4cCI6MTU5MDcxMzcxNywiYXVkIjoid3d3LmV4YW1wbGUuY29tIiwic3ViIjoiMTIzNDU2Nzg5MEBlbWFpbC5jb20iLCJlbWFpbCI6IjEyMzQ1Njc4OTBAZW1haWwuY29tIiwianNtX2lkZW50aXR5IjoiZXlKMGVYQWlPaUpLVjFRaUxDSmhiR2NpT2lKSVV6STFOaUo5LmV5SnBjM01pT2lKUGJteHBibVVnU2xkVUlFSjFhV3hrWlhJaUxDSnBZWFFpT2pFMU5Ua3hOemMzTVRjc0ltVjRjQ0k2TVRVNU1EY3hNemN4Tnl3aVlYVmtJam9pZDNkM0xtVjRZVzF3YkdVdVkyOXRJaXdpYzNWaUlqb2lhbkp2WTJ0bGRFQmxlR0Z0Y0d4bExtTnZiU0lzSW1WdFlXbHNJam9pTVRJek5EVTJOemc1TUVCbGJXRnBiQzVqYjIwaUxDSnliMnhsY3lJNld5SkVaWFlpTENKUWNtOXFaV04wSUVGa2JXbHVhWE4wY21GMGIzSWlYWDAuck5aMndtdTdpYzc3WWR6eFpUNzRnUzlPdDNhbkJWSERNNklvUHdBN2tQYyIsInl1bnRpYW5kdSI6ImV5SjBlWEFpT2lKS1YxUWlMQ0poYkdjaU9pSklVekkxTmlKOS5leUpwYzNNaU9pSlBibXhwYm1VZ1NsZFVJRUoxYVd4a1pYSWlMQ0pwWVhRaU9qRTFOVGt4TmprME16UXNJbVY0Y0NJNk1UVTVNRGN3TlRRek5Dd2lZWFZrSWpvaWQzZDNMbVY0WVcxd2JHVXVZMjl0SWl3aWMzVmlJam9pTVRJek5EVTJOemc1TUVCbGJXRnBiQzVqYjIwaUxDSm1hWEp6ZEU1aGJXVWlPaUptYVhKemRGOXVZVzFsSWl3aVUzVnlibUZ0WlNJNklteGhjM1JmYm1GdFpTSXNJa1Z0WVdsc0lqb2lNVEl6TkRVMk56ZzVNRUJsYldGcGJDNWpiMjBpTENKU2IyeGxJanBiSWtSbGRpSXNJbEJ5YjJwbFkzUWdRV1J0YVc1cGMzUnlZWFJ2Y2lKZGZRLnVUbjU2cW1WTHlETTJYRURYZDU3OXJYY3lrNm1fSHNvM2x0TG9Wd0Q5TGcifQ.qsTfBbaDOAxKJbfR0rJ5nlr9wt4hZzJPco4mjnusu7E"

    def test_current_jwt_token_should_return_none_when_local_threading_dont_have_key(self):

        try:
            delattr(local_threading, "authorization_token")
        except:
            pass

        self.assertIsNone(current_jwt_token())

    def test_current_jwt_token_should_return_none_when_local_threading_have_key(self):

        setattr(local_threading, "authorization_token", self.jwt)

        self.assertIsNotNone(current_jwt_token())
        self.assertEqual(current_jwt_token(), self.jwt)

    def test_get_jsm_token_should_return_none_when_current_jwt_token_return_none(self):

        try:
            delattr(local_threading, "authorization_token")
        except:
            pass

        self.assertIsNone(get_jsm_token())

    def test_get_jsm_token_should_return_token_when_current_jwt_token_have_token(self):

        token_jsm = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJPbmxpbmUgSldUIEJ1aWxkZXIiLCJpYXQiOjE1NTkxNzc3MTcsImV4cCI6MTU5MDcxMzcxNywiYXVkIjoid3d3LmV4YW1wbGUuY29tIiwic3ViIjoianJvY2tldEBleGFtcGxlLmNvbSIsImVtYWlsIjoiMTIzNDU2Nzg5MEBlbWFpbC5jb20iLCJyb2xlcyI6WyJEZXYiLCJQcm9qZWN0IEFkbWluaXN0cmF0b3IiXX0.rNZ2wmu7ic77YdzxZT74gS9Ot3anBVHDM6IoPwA7kPc"

        setattr(local_threading, "authorization_token", self.jwt)

        self.assertIsNotNone(get_jsm_token())
        self.assertEqual(get_jsm_token(), token_jsm)

    def test_get_ltm_token_should_return_none_when_current_jwt_token_return_none(self):

        try:
            delattr(local_threading, "authorization_token")
        except:
            pass

        self.assertIsNone(get_ltm_token())

    def test_get_ltm_token_token_should_return_token_when_current_jwt_token_have_token(self):

        token_ltm = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJPbmxpbmUgSldUIEJ1aWxkZXIiLCJpYXQiOjE1NTkxNjk0MzQsImV4cCI6MTU5MDcwNTQzNCwiYXVkIjoid3d3LmV4YW1wbGUuY29tIiwic3ViIjoiMTIzNDU2Nzg5MEBlbWFpbC5jb20iLCJmaXJzdE5hbWUiOiJmaXJzdF9uYW1lIiwiU3VybmFtZSI6Imxhc3RfbmFtZSIsIkVtYWlsIjoiMTIzNDU2Nzg5MEBlbWFpbC5jb20iLCJSb2xlIjpbIkRldiIsIlByb2plY3QgQWRtaW5pc3RyYXRvciJdfQ.uTn56qmVLyDM2XEDXd579rXcyk6m_Hso3ltLoVwD9Lg"

        setattr(local_threading, "authorization_token", self.jwt)

        self.assertIsNotNone(get_ltm_token())
        self.assertEqual(get_ltm_token(), token_ltm)

    def test_get_jsm_user_data_from_jwt_should_return_none_when_local_threading_dont_have_key(self):

        try:
            delattr(local_threading, "authorization_token")
        except:
            pass

        self.assertIsNone(get_jsm_user_data_from_jwt())

    def test_get_jsm_user_data_from_jwt_should_return_data_when_local_threading_have_key(self):

        data = {
            "iss": "Online JWT Builder",
            "iat": 1559177717,
            "exp": 1590713717,
            "aud": "www.example.com",
            "sub": "jrocket@example.com",
            "email": "1234567890@email.com",
            "roles": ["Dev", "Project Administrator"],
        }

        setattr(local_threading, "authorization_token", self.jwt)

        self.assertIsNotNone(get_jsm_user_data_from_jwt())
        self.assertEqual(get_jsm_user_data_from_jwt(), data)

    def test_get_ltm_user_data_from_jwt_should_return_none_when_local_threading_dont_have_key(self):

        try:
            delattr(local_threading, "authorization_token")
        except:
            pass

        self.assertIsNone(get_ltm_user_data_from_jwt())

    def test_get_ltm_user_data_from_jwt_should_return_data_when_local_threading_have_key(self):

        data = {
            "iss": "Online JWT Builder",
            "iat": 1559169434,
            "exp": 1590705434,
            "aud": "www.example.com",
            "sub": "1234567890@email.com",
            "firstName": "first_name",
            "Surname": "last_name",
            "Email": "1234567890@email.com",
            "Role": ["Dev", "Project Administrator"],
        }

        setattr(local_threading, "authorization_token", self.jwt)

        self.assertIsNotNone(get_ltm_user_data_from_jwt())
        self.assertEqual(get_ltm_user_data_from_jwt(), data)

    def test_get_email_from_jwt_should_return_email_when_token_exists(self):

        setattr(local_threading, "authorization_token", self.jwt)

        self.assertEqual(get_user_email_from_jwt(), "1234567890@email.com")

    @mock.patch("jsm_user_services.services.user.get_jsm_user_data_from_jwt")
    def test_jwt_has_required_roles_assert_all_true(self, mocked_get_jsm_user_data_from_jwt):
        mocked_get_jsm_user_data_from_jwt.return_value = {
            "iss": "Online JWT Builder",
            "iat": 1559177717,
            "exp": 1590713717,
            "aud": "www.example.com",
            "sub": "jrocket@example.com",
            "email": "1234567890@email.com",
            "roles": ["Dev", "Project Administrator"],
        }

        self.assertTrue(jwt_has_required_roles(["Dev", "Project Administrator"]))

    @mock.patch("jsm_user_services.services.user.get_jsm_user_data_from_jwt")
    def test_jwt_has_required_roles_assert_all_false(self, mocked_get_jsm_user_data_from_jwt):
        mocked_get_jsm_user_data_from_jwt.return_value = {
            "iss": "Online JWT Builder",
            "iat": 1559177717,
            "exp": 1590713717,
            "aud": "www.example.com",
            "sub": "jrocket@example.com",
            "email": "1234567890@email.com",
            "roles": ["Dev", "Project Administrator"],
        }

        self.assertFalse(jwt_has_required_roles(["Dev"]))

    @mock.patch("jsm_user_services.services.user.get_jsm_user_data_from_jwt")
    def test_jwt_has_required_roles_assert_any_true(self, mocked_get_jsm_user_data_from_jwt):
        mocked_get_jsm_user_data_from_jwt.return_value = {
            "iss": "Online JWT Builder",
            "iat": 1559177717,
            "exp": 1590713717,
            "aud": "www.example.com",
            "sub": "jrocket@example.com",
            "email": "1234567890@email.com",
            "roles": ["Dev", "Project Administrator"],
        }

        self.assertTrue(jwt_has_required_roles(["Dev"], assert_all=False))

    @mock.patch("jsm_user_services.services.user.get_jsm_user_data_from_jwt")
    def test_jwt_has_required_roles_assert_any_false(self, mocked_get_jsm_user_data_from_jwt):
        mocked_get_jsm_user_data_from_jwt.return_value = {
            "iss": "Online JWT Builder",
            "iat": 1559177717,
            "exp": 1590713717,
            "aud": "www.example.com",
            "sub": "jrocket@example.com",
            "email": "1234567890@email.com",
            "roles": ["Dev", "Project Administrator"],
        }

        self.assertFalse(jwt_has_required_roles(["X"], assert_all=False))



import logging

from django.http import JsonResponse
from jsm_user_services.services.user import current_jwt_token
from jsm_user_services.services.user import get_jsm_token
from jsm_user_services.services.user import get_jsm_user_data_from_jwt
from jsm_user_services.services.user import get_ltm_token
from jsm_user_services.services.user import get_ltm_user_data_from_jwt
from jsm_user_services.services.user import get_user_email_from_jwt


def view_test(request):
    logging.info("teste log")
    data = {
        "current_jwt_token": current_jwt_token(),
        "get_jsm_token": get_jsm_token(),
        "get_ltm_token": get_ltm_token(),
        "get_jsm_user_data_from_jwt": get_jsm_user_data_from_jwt(),
        "get_ltm_user_data_from_jwt": get_ltm_user_data_from_jwt(),
        "get_user_email_from_jwt": get_user_email_from_jwt(),
    }

    return JsonResponse(data)



"""
Django settings for project_jsm_user_services project.

Generated by 'django-admin startproject' using Django 2.2.2.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = ")&0e14na(z=oq_bdqm4fc$0i86l4%z4^zmghowwv+vd-%nduf-"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]


# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "jsm_user_services",
    "app_test",
    "request_id_django_log",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "jsm_user_services.middleware.JsmJwtService",
    "request_id_django_log.middleware.RequestIdDjangoLog",
]

ROOT_URLCONF = "project_jsm_user_services.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

WSGI_APPLICATION = "project_jsm_user_services.wsgi.application"


# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": os.path.join(BASE_DIR, "db.sqlite3")}}


# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {"NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"},
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
    {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
]


# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = "/static/"


USER_API_HOST = "http://aa99c834996a911e998740a5df9f6c11-21eeb1c69c1c7a0b.elb.us-east-1.amazonaws.com/api/v1/"

REQUEST_ID_CONFIG = {
    "REQUEST_ID_HEADER": "HTTP_REQUEST_ID",
    "GENERATE_REQUEST_ID_IF_NOT_FOUND": True,
    "RESPONSE_HEADER_REQUEST_ID": "HTTP_X_REQUEST_ID",
}


LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "filters": {"request_id": {"()": "request_id_django_log.filters.RequestIDFilter"}},
    "formatters": {"standard": {"format": "%(levelname)-8s [%(asctime)s] [%(request_id)s] %(name)s: %(message)s"}},
    "handlers": {
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "filters": ["request_id"],
            "formatter": "standard",
        }
    },
    "loggers": {"": {"handlers": ["console"], "level": "DEBUG", "propagate": False}},
}
